package main

import (
	"image"
	"image/png"
	"image/draw"
	"os"
	"flag"
)

func read(path string) image.Image {
	// Open file
	file, err := os.Open(path)
	if err != nil {
		panic(err)
	}
	defer file.Close()

	// Decode image
	img, _, err := image.Decode(file)
	if err != nil {
		panic(err)
	}

	return img
}

func write(img image.Image, path string) {
	// Open file
	file, err := os.Create(path)
	if err != nil {
		panic(err)
	}
	defer file.Close()

	// Encode image
	err = png.Encode(file, img)
	if err != nil {
		panic(err)
	}
}

func maptile(tile image.Image, to *image.RGBA, index, w, h int,
		top, right, bottom, left, tr, br, bl, tl bool) {
	var offset int
	var x int = index*w

	// Top-left rectangle
	if left && top {
		if tl {
			// Draw completely connected
			offset = w
		} else {
			// Draw the inner corner
			offset = w*2
		}
	} else if left {
		// Draw join left
		offset = w*3
	} else if top {
		// Draw join top
		offset = w*4
	} else {
		// Draw outer corner
		offset = 0
	}
	draw.Draw(to, image.Rect(x, 0, x + w/2, h/2), tile, image.Pt(offset, 0), draw.Src)

	// Top-right rectangle
	if top && right {
		if tr {
			// Draw completely connected
			offset = w
		} else {
			// Draw the inner corner
			offset = w*2
		}
	} else if top {
		// Draw join top
		offset = w*4
	} else if right {
		// Draw join right
		offset = w*3
	} else {
		// Draw outer corner
		offset = 0
	}
	draw.Draw(to, image.Rect(x + w/2, 0, x + w, h/2), tile, image.Pt(w/2 + offset, 0), draw.Src)

	// Bottom-right rectangle
	if right && bottom {
		if br {
			// Draw completely connected
			offset = w
		} else {
			// Draw the inner corner
			offset = w*2
		}
	} else if right {
		// Draw join right
		offset = w*3
	} else if bottom {
		// Draw join bottom
		offset = w*4
	} else {
		// Draw outer corner
		offset = 0
	}
	draw.Draw(to, image.Rect(x + w/2, h/2, x + w, h), tile, image.Pt(w/2 + offset, h/2), draw.Src)

	// Bottom-left rectangle
	if bottom && left {
		if bl {
			// Draw completely connected
			offset = w
		} else {
			// Draw the inner corner
			offset = w*2
		}
	} else if bottom {
		// Draw join bottom
		offset = w*4
	} else if left {
		// Draw join left
		offset = w*3
	} else {
		// Draw outer corner
		offset = 0
	}
	draw.Draw(to, image.Rect(x, h/2, x + w/2, h), tile, image.Pt(offset, h/2), draw.Src)
}

func drawmap(in image.Image) image.Image {
	// Get size of tile
	w := in.Bounds().Max.X / 5
	h := in.Bounds().Max.Y
	if w != h {
		panic("Tiles are not sqaure")
	}

	// Create empty image
	out := image.NewRGBA(image.Rect(0, 0, w*47, h))

	// Draw map
	var index int = 0
	maptile(in, out, index, w, h, false, false, false, false, false, false, false, false)
	index ++

	maptile(in, out, index, w, h, true, false, false, false, false, false, false, false)
	index ++
	maptile(in, out, index, w, h, false, true, false, false, false, false, false, false)
	index ++
	maptile(in, out, index, w, h, false, false, true, false, false, false, false, false)
	index ++
	maptile(in, out, index, w, h, false, false, false, true, false, false, false, false)
	index ++

	maptile(in, out, index, w, h, true, true, false, false, false, false, false, false)
	index ++
	maptile(in, out, index, w, h, true, true, false, false, true, false, false, false)
	index ++
	maptile(in, out, index, w, h, false, true, true, false, false, false, false, false)
	index ++
	maptile(in, out, index, w, h, false, true, true, false, false, true, false, false)
	index ++
	maptile(in, out, index, w, h, false, false, true, true, false, false, false, false)
	index ++
	maptile(in, out, index, w, h, false, false, true, true, false, false, true, false)
	index ++
	maptile(in, out, index, w, h, true, false, false, true, false, false, false, false)
	index ++
	maptile(in, out, index, w, h, true, false, false, true, false, false, false, true)
	index ++

	maptile(in, out, index, w, h, true, false, true, false, false, false, false, false)
	index ++
	maptile(in, out, index, w, h, false, true, false, true, false, false, false, false)
	index ++

	maptile(in, out, index, w, h, true, true, false, true, false, false, false, false)
	index ++
	maptile(in, out, index, w, h, true, true, false, true, false, false, false, true)
	index ++
	maptile(in, out, index, w, h, true, true, false, true, true, false, false, false)
	index ++
	maptile(in, out, index, w, h, true, true, false, true, true, false, false, true)
	index ++

	maptile(in, out, index, w, h, true, true, true, false, false, false, false, false)
	index ++
	maptile(in, out, index, w, h, true, true, true, false, true, false, false, false)
	index ++
	maptile(in, out, index, w, h, true, true, true, false, false, true, false, false)
	index ++
	maptile(in, out, index, w, h, true, true, true, false, true, true, false, false)
	index ++

	maptile(in, out, index, w, h, false, true, true, true, false, false, false, false)
	index ++
	maptile(in, out, index, w, h, false, true, true, true, false, true, false, false)
	index ++
	maptile(in, out, index, w, h, false, true, true, true, false, false, true, false)
	index ++
	maptile(in, out, index, w, h, false, true, true, true, false, true, true, false)
	index ++

	maptile(in, out, index, w, h, true, false, true, true, false, false, false, false)
	index ++
	maptile(in, out, index, w, h, true, false, true, true, false, false, true, false)
	index ++
	maptile(in, out, index, w, h, true, false, true, true, false, false, false, true)
	index ++
	maptile(in, out, index, w, h, true, false, true, true, false, false, true, true)
	index ++

	maptile(in, out, index, w, h, true, true, true, true, false, false, false, false)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, false, false, false, true)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, true, false, false, false)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, true, false, false, true)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, false, true, false, false)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, false, true, false, true)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, true, true, false, false)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, true, true, false, true)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, false, false, true, false)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, false, false, true, true)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, true, false, true, false)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, true, false, true, true)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, false, true, true, false)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, false, true, true, true)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, true, true, true, false)
	index ++
	maptile(in, out, index, w, h, true, true, true, true, true, true, true, true)
	index ++

	return out
}

func main() {
	var in, out string
	flag.StringVar(&in, "in", "in.png", "File to read in the same format as notes/connectedmap.png")
	flag.StringVar(&out, "out", "out.png", "File to write")
	flag.Parse()

	img := read(in)
	img = drawmap(img)
	write(img, out)
}
