#ifndef __HEIGHTMAP_H__
#define __HEIGHTMAP_H__

/* Heightmap tools for terrain generator. */

#include "config.h"

// Height information, same size as world
typedef struct {
	float data[WORLDSIZE*WORLDSIZE];
} Heightmap;

// Set heightmap to zeros
void heightmap_clear(Heightmap *data);
// Add hump to heightmap
void heightmap_hump(Heightmap *data, const float x, const float y, const float radius, const float height);
// Multiply heightmap by world sized hump
void heightmap_bias(Heightmap *data);
// Convert heightmap to blocks in the current world
// NOTE: Only overwrites air
// Uses the heightmap value times 'multiply' plus 'add'
void heightmap_blocks(const Heightmap *data, const float add, const float multiply, const unsigned short type);
// Return height at x, y
float heightmap_get(const Heightmap *data, const int x, const int y);

// Density information, same size as world
// This is a 3D version of Heightmap
typedef struct {
	float data[WORLDSIZE*WORLDSIZE*WORLDSIZE];
} Densitymap;

// Set densitymap to zeros
void densitymap_clear(Densitymap *data);
// Add blob to densitymap
void densitymap_blob(Densitymap *data, const float x, const float y, const float z, const float radius, const float weight);
// Multiply densitymap by world sized blob
void densitymap_bias(Densitymap *data);
// Convert densitymap to blocks in the current world
// NOTE: Only overwrites air
// Uses the densitymap data time 'multiply' plus 'add' offset upward by 'offset' blocks
// Blocks are added where the resulting density is more than one
void densitymap_blocks(
	const Densitymap *data,
	const float add,
	const float multiply,
	const int offset,
	const unsigned short type);

#endif

