/* Provides functions for rendering the world. */

#ifndef __RENDER_H__
#define __RENDER_H__

#include "resource.h"
#include "world.h"

#define TEXTURESIZE 16

/* Special voxel flags. */
enum {
	// Use texture+1 if a face is next to solid blocks on every side
	RENDER_JOIN = 1,
	// Draw flat cross, Minecraft plant shape
	RENDER_X = 2,
	// Fully connected texture
	RENDER_CONNECTED = 3,
	// Item on flat surface
	RENDER_TOOL = 4,
};

/* Textures to use for types of voxel. */
extern unsigned int topMap[VOXELTYPES];
extern unsigned int bottomMap[VOXELTYPES];
extern unsigned int leftMap[VOXELTYPES];
extern unsigned int rightMap[VOXELTYPES];
extern unsigned int frontMap[VOXELTYPES];
extern unsigned int backMap[VOXELTYPES];
extern unsigned char specialMap[VOXELTYPES];

/* Setup OpenGL. Must be called after loading resources. */
void render_init();
/* Resize viewport. */
void render_resize();
/* Generates sky cube.
Must be called after render_init.
Must be called once before rendering, can be called repeatedly. */
void render_updatesky(const unsigned long horizoncolour, const unsigned long atmospherecolour, const unsigned long zenithcolour);
/* Request that the VBO be rebuilt this frame. */
void render_update();
/* Redraw.
x, y, z are the position of the camera.
pan, tilt are the angle of the camera.
selectionValid is true if the player is looking at a selectable voxel.
sx, sy, sz are the coordinates of the voxel the player is looking at.
dx, dy, dz are the dirction of the face of the selected voxel beneath the cursor.
tool is the type of voxel that is being used, which is displayed within the HUD. */
void render_normal(
	const float x, const float y, const float z, const float pan, const float tilt,
	const char selectionValid,
	const int sx, const int sy, const int sz,
	const char dx, const char dy, const char dz,
	const unsigned short tool);

#endif
